#!/usr/bin/fades

import datetime
import pathlib
import sys
import uuid

import exifread  # fades

EXIF_DATE_FORMAT = "%Y:%m:%d %H:%M:%S"


def _get_date(imagepath, deltas):
    """Get the date from a photo."""
    with open(imagepath, "rb") as fh:
        metadata = exifread.process_file(fh)
    dt = metadata.get('EXIF DateTimeOriginal')
    if dt is None:
        return None
    tstamp = datetime.datetime.strptime(str(dt), EXIF_DATE_FORMAT).timestamp()
    for prefix, delta in deltas:
        if imagepath.name.startswith(prefix):
            tstamp += delta
    new_dt = datetime.datetime.fromtimestamp(tstamp)
    return new_dt


def _build_name(basename, add_item):
    return basename[:-4] + add_item + basename[-4:]


def main(directory, deltas, nodate):
    """Main entry point."""
    # get all the files in the directoryory, with their timestamps
    photos = []
    print("Analyzing files")
    for fpath in directory.iterdir():
        tstamp = _get_date(fpath, deltas)

        if tstamp is None and not nodate:
            print("    Ignoring", repr(fpath))
            continue

        # build new name
        if tstamp is None:
            newname = f"sinfecha-{fpath.name}"
        else:
            unique = str(uuid.uuid4())[:5]
            newname = "cam-{0:%Y%m%d}-{0:%H%M%S}-{1}.jpeg".format(tstamp, unique)

        photos.append((fpath, newname))

    print("Found {} photos".format(len(photos)))

    for fpath, newname in sorted(photos):
        fpath.rename(directory / newname)

    print("Done.")


USAGE = """
Usage ,rename_mix_photos.py directory [[prefix=delta] ...]

    Get all photos in a directory (mixed from camera and phones) and rename
    them according to their dates (so name-ordering gets chronological).

    Optionally several `prefix=delta` can be included, delta should be in
    the form of +N or -N, being N the hours to add/sub from images starting
    with the indicated prefix.

    If --nodate is given, the media without datetime information
    will still be renamed just adding a prefix.
"""


def _parse_args():
    if "--nodate" in sys.argv:
        sys.argv.remove("--nodate")
        nodate = True
    else:
        nodate = False

    _, directory, *rawdeltas = sys.argv
    deltas = []
    for rawdelta in rawdeltas:
        prefix, rawdelta = rawdelta.split("=")
        if rawdelta[0] == '+':
            mult = 3600
        elif rawdelta[0] == '-':
            mult = -3600
        else:
            raise ValueError()
        delta = mult * int(rawdelta[1:])
        deltas.append((prefix, delta))

    return directory, deltas, nodate


if __name__ == "__main__":
    try:
        directory, deltas, nodate = _parse_args()
    except Exception:
        print(USAGE)
    else:
        main(pathlib.Path(directory), deltas, nodate)
